@@ -138,6 +138,9 @@ wrapper_poppler_page_render_to_pixbuf (PopplerPage *page,
 			       int rotation,
 			       GdkPixbuf *pixbuf);
 
+// wrapper for gtk_dialog_run that disables xinput (bug #159)
+gint wrapper_gtk_dialog_run(GtkDialog *dialog);
+
 // defines for paper rulings
 
 #define RULING_MARGIN_COLOR 0xff0080ff
